#!/usr/bin/env node

var fs = require('fs');
var common = require('common');

var mud = require('../mud');

var argv = function() {
	var options = {};
	var args = process.argv.slice(2);

	var arg = function(key, fn) {
		if (Array.isArray(key)) {
			key.forEach(function(k) {
				arg(k, fn);
			});
			return;
		}
		var regex = Object.prototype.toString.call(key) === '[object RegExp]';
		
		for (var i = 0; i < args.length; i++) {
			if (regex ? key.test(args[i]) : args[i] === key) {
				fn(regex ? args[i] : args[i+1], key);
				return;
			}
			if (args[i].indexOf(key+'=') === 0) {
				fn(args[i].substring(key.length+1), key);
				return;
			}
		}
	};

	arg(['help', 'list', 'ls', 'global'], function(_, key) {
		options[key === 'ls' ? 'list' : key] = true;
	});
	arg('source', function(source) {
		options.source = source;
	})
	arg('publish', function(filename) {
		options.publish = fs.realpathSync(filename);
	});
	arg('server', function(value) {
		options.server = /^\d+$/.test(value) ? parseInt(value, 10) : 10000;
	});
	arg(['install', 'modules'], function(value, key) {
		options[key] = value ? value.split(',') : [];
	});
	arg('compile', function(value) {
		options.compile = {'simple':'simple', 'advanced':'advanced'}[value] || 'simple';		
	});
	arg(/\.js$/i, function(filename) {
		options.filename = fs.realpathSync(filename);
	});

	return options;
}();

var stack = function(err) {
	console.error(err.stack);
};

if (argv.list) {
	mud.list(common.fork(stack, function(mods) {
		console.log((mods || []).join('\n'));
	}));
	return;
}
if (argv.server) {
	require('../mud-server.js');
	return;
}
if (argv.modules || argv.filename || argv.source) {
	mud.resolve(argv, common.fork(stack, function(src) {
		console.log(src);
	}));
	return;
}

console.error('usage: mud [option] [filename]\n');
console.error('where the options are:');
console.error('  install   name  - to install new packages');
console.error('  uninstall name  - remove package');
console.error('  list            - list all installed packages');
console.error('  compile   mode? - compile the code using the Google Closure Compiler');
console.error('  server    port  - run a mud server');
console.error('  global    list  - bind modules to global vars');
console.error('  modules   list  - load in these modules');
console.error('  source    code  - resolve this code');
console.error('  publish   file  - publish your own package');